@@ -3,78 +3,73 @@ import signal
 
 import pytest
 from snakeoil import process
+from snakeoil.contexts import chdir
 from snakeoil.process import spawn
-from snakeoil.test.fixtures import TempDir
 
 BASH_BINARY = process.find_binary("bash", fallback='')
 
 @pytest.mark.skipif(not BASH_BINARY, reason='missing bash binary')
-class TestSpawn(TempDir):
+class TestSpawn:
 
-    def setup(self):
-        self.orig_env = os.environ["PATH"]
-        self.null_file = open("/dev/null", "w")
-        self.null = self.null_file.fileno()
-        os.environ["PATH"] = ":".join([self.dir] + self.orig_env.split(":"))
+    @pytest.fixture(autouse=True)
+    def _setup(self, tmp_path):
+        orig_path = os.environ["PATH"]
+        os.environ["PATH"] = ":".join([str(tmp_path)] + orig_path.split(":"))
 
-    def teardown(self):
-        self.null_file.close()
-        os.environ["PATH"] = self.orig_env
+        yield
 
-    def generate_script(self, filename, text):
-        if not os.path.isabs(filename):
-            fp = os.path.join(self.dir, filename)
-        with open(fp, "w") as f:
-            f.write("#!/usr/bin/env bash\n")
-            f.write(text)
-        os.chmod(fp, 0o750)
-        assert os.stat(fp).st_mode & 0o750 == 0o750
+        os.environ["PATH"] = orig_path
+
+    @pytest.fixture
+    def dev_null(self):
+        """Return an open write file descriptor to `/dev/null`."""
+        with open("/dev/null", "w") as null_file:
+            yield null_file.fileno()
+
+    def generate_script(self, tmp_path, filename, text):
+        assert not os.path.isabs(filename)
+        fp = tmp_path / filename
+        fp.write_text("#!/usr/bin/env bash\n" + text)
+        fp.chmod(0o750)
+        assert fp.stat().st_mode & 0o750 == 0o750
         return fp
 
-    def test_get_output(self):
+    def test_get_output(self, tmp_path, dev_null):
         filename = "spawn-getoutput.sh"
         for r, s, text, args in (
                 [0, ["dar\n"], "echo dar\n", {}],
                 [0, ["dar"], "echo -n dar", {}],
                 [1, ["blah\n", "dar\n"], "echo blah\necho dar\nexit 1", {}],
-                [0, [], "echo dar 1>&2", {"fd_pipes": {1: 1, 2: self.null}}]):
-            fp = self.generate_script(filename, text)
-            assert (r, s) == spawn.spawn_get_output(fp, spawn_type=spawn.spawn_bash, **args)
+                [0, [], "echo dar 1>&2", {"fd_pipes": {1: 1, 2: dev_null}}]):
+            fp = self.generate_script(tmp_path, filename, text)
+            assert (r, s) == spawn.spawn_get_output(str(fp), spawn_type=spawn.spawn_bash, **args)
         os.unlink(fp)
 
     @pytest.mark.skipif(not spawn.is_sandbox_capable(), reason="missing sandbox binary")
-    def test_sandbox(self):
+    def test_sandbox(self, tmp_path):
         fp = self.generate_script(
-            "spawn-sandbox.sh", "echo $LD_PRELOAD")
-        ret = spawn.spawn_get_output(fp, spawn_type=spawn.spawn_sandbox)
+            tmp_path, "spawn-sandbox.sh", "echo $LD_PRELOAD")
+        ret = spawn.spawn_get_output(str(fp), spawn_type=spawn.spawn_sandbox)
         assert ret[1], "no output; exit code was %s; script location %s" % (ret[0], fp)
         assert "libsandbox.so" in [os.path.basename(x.strip()) for x in ret[1][0].split()]
         os.unlink(fp)
 
     @pytest.mark.skipif(not spawn.is_sandbox_capable(), reason="missing sandbox binary")
-    def test_sandbox_empty_dir(self):
+    def test_sandbox_empty_dir(self, tmp_path):
         """sandbox gets pissy if it's ran from a nonexistent dir
 
         this verifies our fix works.
         """
         fp = self.generate_script(
-            "spawn-sandbox.sh", "echo $LD_PRELOAD")
-        dpath = os.path.join(self.dir, "dar")
-        os.mkdir(dpath)
-        try:
-            cwd = os.getcwd()
-        except OSError:
-            cwd = None
-        try:
-            os.chdir(dpath)
-            os.rmdir(dpath)
+            tmp_path, "spawn-sandbox.sh", "echo $LD_PRELOAD")
+        dpath = tmp_path / "dar"
+        dpath.mkdir()
+        with chdir(dpath):
+            dpath.rmdir()
             assert "libsandbox.so" in \
                 [os.path.basename(x.strip()) for x in spawn.spawn_get_output(
-                    fp, spawn_type=spawn.spawn_sandbox, cwd='/')[1][0].split()]
-            os.unlink(fp)
-        finally:
-            if cwd is not None:
-                os.chdir(cwd)
+                    str(fp), spawn_type=spawn.spawn_sandbox, cwd='/')[1][0].split()]
+            fp.unlink()
 
     def test_process_exit_code(self):
         assert spawn.process_exit_code(0) == 0
@@ -104,12 +99,12 @@ class TestSpawn(TempDir):
     def test_spawn_bash(self, capfd):
         # bash builtin for true without exec'ing true (eg, no path lookup)
         assert 0 == spawn.spawn_bash('echo bash')
-        out, err = capfd.readouterr()
+        out, _err = capfd.readouterr()
         assert out.strip() == 'bash'
 
-    def test_umask(self):
+    def test_umask(self, tmp_path):
         fp = self.generate_script(
-            "spawn-umask.sh", "#!%s\numask" % BASH_BINARY)
+            tmp_path, "spawn-umask.sh", f"#!{BASH_BINARY}\numask")
         try:
             old_umask = os.umask(0)
             if old_umask == 0:
@@ -119,6 +114,6 @@ class TestSpawn(TempDir):
             else:
                 desired = 0
             assert str(desired).lstrip("0") == \
-                spawn.spawn_get_output(fp)[1][0].strip().lstrip("0")
+                spawn.spawn_get_output(str(fp))[1][0].strip().lstrip("0")
         finally:
             os.umask(old_umask)
